<?php

namespace Drupal\hubspot_integration\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\hubspot_integration\Services\HubspotAPI;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\taxonomy\Entity\Term;

/**
 * Configures Hubspot mapping for this site.
 */
class HubspotMappingForm extends ConfigFormBase {
  protected $config_factory = NULL;
  /** @var \Drupal\hubspot_integration\Services\HubspotAPI $hubspot_api */
  protected $hubspot_api = NULL;
  protected $properties = [];
  protected $mapping = [];
  protected $managed_types = ['enumeration'];

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('hubspot_integration.api')
    );
  }

  /**
   * Constructs a \Drupal\HubspotIntegration\Form\HubspotAdminForm object.
   *
   */
  public function __construct(ConfigFactoryInterface $config_factory, HubspotAPI $hubspot_api) {
    $this->config_factory = $config_factory;
    $this->hubspot_api = $hubspot_api;

    $properties = $this->hubspot_api->getContactProperties();
    foreach($properties as $property) {
      $this->properties[$property->name] = $property;
    }

    $this->mapping = $this->hubspot_api->getMapping();

    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'hubspot_integration_admin_mapping_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['hubspot_integration.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $settings = $this->config('hubspot_integration.settings');

    if (empty($settings->get('api_key'))) {
      \Drupal::messenger()->addError('No API Key');
      return $form;
    }

    if ($form_state->isSubmitted() || !$form_state->isValueEmpty($key)) {
      $this->mapping = HubspotMappingForm::getMappingFromFormState($form_state);
    }

    $options = [];
    foreach($this->properties as $property_name => $property) {
      if (in_array($property->type, $this->managed_types)) {
        $options[$property_name] = $property->label .' - ('.$property->type.')';
      }
    }

    $form['container'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'select-container'],
    ];

    $form['container']['property_select'] = [
      '#type' => 'select',
      '#title' => 'Select a property',
      '#options' => $options,
      '#default_value' => NULL,
    ];

    $form['container']['add_item'] = [
      '#type'   => 'button',
      '#value'  => $this->t('Add another item'),
      '#name' => 'add_item',
      '#ajax'   => [
        'callback' => '::addProperty',
        'wrapper'  => 'mapping-container',
      ],
    ];

    $form['mapping'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'mapping-container'],
    ];

    foreach($this->mapping as $property_name => $mapping) {
      $property = $this->properties[$property_name];

      $form['mapping']['details__'.$property->name] = [
        '#type' => 'details',
        '#title' => $property->label.' ('.$property->type.')',
        '#description' => $property->description,
        '#open' => FALSE,
      ];

      $form['mapping']['details__'.$property_name]+=$this->mappingItemForm($property, $form_state, $mapping);
    }

    if ($triggering_element = $form_state->getUserInput()['_triggering_element_name']) {

      if ($triggering_element == 'add_item') {
        $property_name = $form_state->getValue('property_select');
      }
      elseif (stristr($triggering_element, 'vocabulary__')) {
        $property_name = str_replace('vocabulary__', '', $form_state->getUserInput()['_triggering_element_name']);
      }

      $property = $this->properties[$property_name];

      $form['mapping']['details__'.$property->name] = [
        '#type' => 'details',
        '#title' => $property->label.' ('.$property->type.')',
        '#description' => $property->description,
        '#open' => TRUE,
      ];

      $form['mapping']['details__'.$property_name]+=$this->mappingItemForm($property, $form_state);

      $form_state->setRebuild();
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * Build a form item depending on hubspot item type
   */
  protected function mappingItemForm($property, FormStateInterface $form_state, $mapping = NULL) {
    $item = [];
    switch($property->type) {
      case 'enumeration':
        $item = $this->mappingItemEnumerationForm($property, $form_state, $mapping);
        break;
      case 'bool': //TODO : handler ?
      case 'string': //TODO : handler ?
      case 'phone_number': //TODO : handler ?
      case 'datetime': //TODO : handler ?
      default:
    }

    return $item;
  }

  /**
   * Build an taxonomy mapping form item (Hubspot Enumeration)
   */
  protected function mappingItemEnumerationForm($property, FormStateInterface $form_state, $mapping = NULL) {
    $item_opts = [];


    $vocabularies = Vocabulary::loadMultiple();
    $vocabulariesList = [];

    foreach ($vocabularies as $vid => $vocablary) {
      $vocabulariesList[$vid] = $vocablary->get('name');
    }

    $item_opts['vocabulary__'.$property->name] = [
      '#type' => 'select',
      '#title' => $this->t('Please select a vocabulary to map'),
      '#options' => [0=>$this->t('-- None --')]+$vocabulariesList,
      '#default_value' => $this->mapping[$property->name]['#vocabulary'],
      '#ajax' => [
        'callback' => '::populateMappingOptions',
        'wrapper' => 'mapping__'.$property->name,
        'event' => 'change',
        'method' => 'replace',
        'effect' => 'fade',
      ],
    ];

    $item_opts['mapping__'.$property->name] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'mapping__'.$property->name],
    ];

    foreach($property->options as $option) {
      //Hubspot label & values are human readable, we have to create a machine name to avoid the forms items key issues
      $option->value = HubspotAPI::machineName($option->value);

      $item_opts['mapping__'.$property->name]['hubspot__'.$property->name.'__'.$option->value] = [
        '#type' => 'entity_autocomplete',
        '#target_type' => 'taxonomy_term',
        '#title' => $option->label,
        '#description' => $option->description,
        '#default_value' => (isset($this->mapping[$property->name]['#terms'][$option->value]))?Term::load($this->mapping[$property->name]['#terms'][$option->value]):NULL,
        '#tags' => FALSE,
        '#selection_settings' => [
          'target_bundles' => [(isset($this->mapping[$property->name]['#vocabulary']))?$this->mapping[$property->name]['#vocabulary']:NULL],
        ],
        '#weight' => '0',
      ];
    }

    return $item_opts;
  }

  /**
   * Ajax callback when selecting a mapping option
   *
   * @param array $form
   * @param FormStateInterface $form_state
   * @return unknown
   */
  public function populateMappingOptions(array &$form, FormStateInterface $form_state) {
    $property_name = str_replace('vocabulary__', '', $form_state->getUserInput()['_triggering_element_name']);
    return $form['mapping']['details__'.$property_name]['mapping__'.$property_name];
  }

  /**
   * Ajax callback when add a new mapping item form
   *
   * @param array $form
   * @param FormStateInterface $form_state
   * @return unknown
   */
  public function addProperty(array &$form, FormStateInterface $form_state) {
    return $form['mapping'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $this->mapping = HubspotMappingForm::getMappingFromFormState($form_state);
    $config = $this->config('hubspot_integration.settings');
    $config->set('mapping', $this->mapping);
    $config->save();
  }


  protected static function getMappingFromFormState(FormStateInterface $form_state) {
    $mapping = [];

    foreach($form_state->getValues() as $key => $value) {
      if (stristr($key, 'vocabulary__')) {
        $property_str = str_replace('vocabulary__', '', $key);
        $arr = explode('__', $property_str);
        $property_name = $arr[0];
        $mapping[$property_name]['#vocabulary']=$value;
      }
      elseif (stristr($key, 'hubspot__')) {
        $property_str = str_replace('hubspot__', '', $key);
        $arr = explode('__', $property_str);
        $property_name = $arr[0];
        $option_value = $arr[1];
        $mapping[$property_name]['#terms'][$option_value]=$value;
      }
    }

    return $mapping;
  }
}
